@@ -85,7 +85,7 @@ TPLINK_KELVIN = {
     "KB130": ColorTempRange(2500, 9000),
     "KL130": ColorTempRange(2500, 9000),
     "KL125": ColorTempRange(2500, 6500),
-    "KL135": ColorTempRange(2500, 6500),
+    "KL135": ColorTempRange(2500, 9000),
     r"KL120\(EU\)": ColorTempRange(2700, 6500),
     r"KL120\(US\)": ColorTempRange(2700, 5000),
     r"KL430": ColorTempRange(2500, 9000),
@@ -219,7 +219,7 @@ class IotBulb(IotDevice):
         self.add_module(
             Module.IotAntitheft, Antitheft(self, "smartlife.iot.common.anti_theft")
         )
-        self.add_module(Module.IotTime, Time(self, "smartlife.iot.common.timesetting"))
+        self.add_module(Module.Time, Time(self, "smartlife.iot.common.timesetting"))
         self.add_module(Module.Energy, Emeter(self, self.emeter_type))
         self.add_module(Module.IotCountdown, Countdown(self, "countdown"))
         self.add_module(Module.IotCloud, Cloud(self, "smartlife.iot.common.cloud"))
@@ -367,7 +367,9 @@ class IotBulb(IotDevice):
         saturation = light_state["saturation"]
         value = self._brightness
 
-        return HSV(hue, saturation, value)
+        # Simple HSV(hue, saturation, value) is less efficent than below
+        # due to the cpython implementation.
+        return tuple.__new__(HSV, (hue, saturation, value))
 
     @requires_update
     async def _set_hsv(
